/**************************************************************************
*  Copyright (c) 2013-2015 by Michael Fischer.
*  All rights reserved.
*
*  Based on an example from ST. Therefore partial copyright:
*  Copyright (c) 2011 STMicroelectronics
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  07.06.2013  mifi  First Version.
*  19.08.2013  mifi  Some rework and cleanup.
*  18.04.2014  mifi  Support STM32F2x7 and STM32F4x7.
*  08.06.2015  mifi  Change ETH interrupt priority from 2 to 3.
**************************************************************************/
#define __ETH_CONFIG_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <stdint.h>
#include <string.h>

#include "project.h"
#include "lwip/opt.h"
#include "ip_phy_driver.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

   /* STM324xG-EVAL jumpers setting
    +===================================================+
    +  Jumper |       MII mode configuration            +
    +===================================================+
    +  JP5    | 2-3 provide 25MHz clock by MCO(PA8)     +
    +         | 1-2 provide 25MHz clock by ext. Crystal +
    + --------------------------------------------------+
    +  JP6    |          2-3                            +
    + --------------------------------------------------+
    +  JP8    |          Open                           +
    +===================================================+
    */

/* PHY registers */
#define PHYIDR1   0x02
#define PHYIDR2   0x03


#if defined(USE_BOARD_MB786)
#define GPIO_CONFIG  GPIO_Config_MB786
#endif /* USE_BOARD_MB786 */

#if defined(USE_BOARD_E407)
#define GPIO_CONFIG  GPIO_Config_E407
#endif /* USE_BOARD_E407 */

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static IP_PHY_DRIVER PHYDriver;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  PHYReadReg                                                           */
/*                                                                       */
/*  Read a register of the PHY.                                          */
/*                                                                       */
/*  In    : pPHY, Reg                                                    */
/*  Out   : none                                                         */
/*  Return: Value of the register                                        */
/*************************************************************************/
static uint16_t PHYReadReg (IP_PHY_DRIVER *pPHY, uint16_t Reg)
{
   return( ETH_ReadPHYRegister(pPHY->Address, Reg) );
} /* PHYReadReg */

/*************************************************************************/
/*  PHYWriteReg                                                          */
/*                                                                       */
/*  Write a register of the PHY.                                         */
/*                                                                       */
/*  In    : pPHY, Reg, Value                                             */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void PHYWriteReg (IP_PHY_DRIVER *pPHY, uint16_t Reg, uint16_t Value)
{
   ETH_WritePHYRegister(pPHY->Address, Reg, Value);
} /* PHYReadReg */

/*************************************************************************/
/*  FindPHY                                                              */
/*                                                                       */
/*  Find the PHY which is used.                                          */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
IP_PHY_DRIVER *FindPHY (void)
{
   uint32_t Value;
   uint8_t  Addr;

   /*
    * Setup PHYDriver
    */
   memset(&PHYDriver, 0x00, sizeof(PHYDriver));    
   PHYDriver.ReadReg  = PHYReadReg;
   PHYDriver.WriteReg = PHYWriteReg;

   /*
    * Search the PHY...
    */
   for (Addr = 0; Addr < 32; Addr++)
   {    
      PHYDriver.Address = Addr; 
    
      /* Reset PHY */
      PHYWriteReg(&PHYDriver, PHY_BCR, PHY_Reset);
      
      /* Give PHY some time to reset */
      TimeDly(2);
    
      Value  = PHYReadReg(&PHYDriver, PHYIDR1);    
      Value  = Value << 16;
      Value |= PHYReadReg(&PHYDriver, PHYIDR2);
      Value &= PHY_ID_MASK;
   
      switch (Value)
      {
         case PHY_ID_DP83848C:
         {
            PHY_DP83848.ReadReg  = PHYReadReg;
            PHY_DP83848.WriteReg = PHYWriteReg;
            PHY_DP83848.Address  = Addr;
            return(&PHY_DP83848);
            break;
         }
         
         case PHY_ID_KSZ8721BL:
         {
            PHY_KSZ8721.ReadReg  = PHYReadReg;
            PHY_KSZ8721.WriteReg = PHYWriteReg;
            PHY_KSZ8721.Address  = Addr;
            return(&PHY_KSZ8721);
            break;
         }
      
         default:
         {
            /* Do noting */
            break;
         }
      }
   }      
      
   return(NULL);
} /* CheckPHY */

/*************************************************************************/
/*  MACDMA_Config                                                        */
/*                                                                       */
/*  Configures the Ethernet Interface.                                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void MACDMA_Config (void)
{
   ETH_InitTypeDef ETH_InitStructure;

   /* Enable ETHERNET clock  */
   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_ETH_MAC | RCC_AHB1Periph_ETH_MAC_Tx |
                          RCC_AHB1Periph_ETH_MAC_Rx, ENABLE);                                             

   /* Reset ETHERNET on AHB Bus */
   ETH_DeInit();

   /* Software reset */
   ETH_SoftwareReset();

  /* Wait for software reset */
   while (ETH_GetSoftwareResetStatus() == SET);

   /* 
    * ETHERNET Configuration
    * Call ETH_StructInit if you don't like to configure
    * all ETH_InitStructure parameter 
    */
   ETH_StructInit(&ETH_InitStructure);

   /*------------------------   MAC   -----------------------------------*/
   ETH_InitStructure.ETH_AutoNegotiation          = ETH_AutoNegotiation_Disable; 
   ETH_InitStructure.ETH_Speed                    = ETH_Speed_10M;
   ETH_InitStructure.ETH_Mode                     = ETH_Mode_FullDuplex;   
   ETH_InitStructure.ETH_LoopbackMode             = ETH_LoopbackMode_Disable;
   ETH_InitStructure.ETH_RetryTransmission        = ETH_RetryTransmission_Disable;
   ETH_InitStructure.ETH_AutomaticPadCRCStrip     = ETH_AutomaticPadCRCStrip_Disable;
   ETH_InitStructure.ETH_ReceiveAll               = ETH_ReceiveAll_Disable;
   ETH_InitStructure.ETH_BroadcastFramesReception = ETH_BroadcastFramesReception_Enable;
   ETH_InitStructure.ETH_PromiscuousMode          = ETH_PromiscuousMode_Disable;
   ETH_InitStructure.ETH_MulticastFramesFilter    = ETH_MulticastFramesFilter_Perfect;
   ETH_InitStructure.ETH_UnicastFramesFilter      = ETH_UnicastFramesFilter_Perfect;
   
#ifdef CHECKSUM_BY_HARDWARE
   ETH_InitStructure.ETH_ChecksumOffload          = ETH_ChecksumOffload_Enable;
#endif

   /*------------------------   DMA   -----------------------------------*/
  
   /* 
    * When we use the Checksum offload feature, we need to enable the Store
    * and Forward mode. The store and forward guarantee that a whole frame is
    * stored in the FIFO, so the MAC can insert/verify the checksum, if the
    * checksum is OK the DMA can handle the frame otherwise the frame is dropped 
    */
   ETH_InitStructure.ETH_DropTCPIPChecksumErrorFrame = ETH_DropTCPIPChecksumErrorFrame_Enable; 
   ETH_InitStructure.ETH_ReceiveStoreForward         = ETH_ReceiveStoreForward_Enable;         
   ETH_InitStructure.ETH_TransmitStoreForward        = ETH_TransmitStoreForward_Enable;     
   ETH_InitStructure.ETH_ForwardErrorFrames          = ETH_ForwardErrorFrames_Disable;       
   ETH_InitStructure.ETH_ForwardUndersizedGoodFrames = ETH_ForwardUndersizedGoodFrames_Disable;   
   ETH_InitStructure.ETH_SecondFrameOperate          = ETH_SecondFrameOperate_Enable;
   ETH_InitStructure.ETH_AddressAlignedBeats         = ETH_AddressAlignedBeats_Enable;      
   ETH_InitStructure.ETH_FixedBurst                  = ETH_FixedBurst_Enable;                
   ETH_InitStructure.ETH_RxDMABurstLength            = ETH_RxDMABurstLength_32Beat;          
   ETH_InitStructure.ETH_TxDMABurstLength            = ETH_TxDMABurstLength_32Beat;
   ETH_InitStructure.ETH_DMAArbitration              = ETH_DMAArbitration_RoundRobin_RxTx_2_1;

   /*
    * Configure Ethernet now 
    */
   ETH_Init(&ETH_InitStructure, 0);

   /* 
    * Enable the Ethernet Rx Interrupt 
    */
   ETH_DMAITConfig(ETH_DMA_IT_NIS | ETH_DMA_IT_R, ENABLE);
} /* MACDMA_Config */

#if defined(USE_BOARD_MB786)
/*************************************************************************/
/*  GPIO_Config_MB786                                                    */
/*                                                                       */
/*  Configures the GPIO ports.                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void GPIO_Config_MB786 (void)
{
   GPIO_InitTypeDef GPIO_InitStructure;

   /* Enable GPIOs clocks */
   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB |
                          RCC_AHB1Periph_GPIOC | RCC_AHB1Periph_GPIOI |
                          RCC_AHB1Periph_GPIOG | RCC_AHB1Periph_GPIOH , ENABLE);

   /* Enable SYSCFG clock */
   RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
 
   /* Configure MCO (PA8) */
   GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_8;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
   GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
   GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL ;
   GPIO_Init(GPIOA, &GPIO_InitStructure);
  
   /* MII Media interface */

   /* Output HSE clock (25MHz) on MCO pin (PA8) to clock the PHY */
   RCC_MCO1Config(RCC_MCO1Source_HSE, RCC_MCO1Div_1);

   SYSCFG_ETH_MediaInterfaceConfig(SYSCFG_ETH_MediaInterface_MII);
  
   /* Ethernet pins configuration
   
      ETH_MDIO -------------------------> PA2
      ETH_MDC --------------------------> PC1
      ETH_PPS_OUT ----------------------> PB5
      ETH_MII_CRS ----------------------> PH2
      ETH_MII_COL ----------------------> PH3
      ETH_MII_RX_ER --------------------> PI10
      ETH_MII_RXD2 ---------------------> PH6
      ETH_MII_RXD3 ---------------------> PH7
      ETH_MII_TX_CLK -------------------> PC3
      ETH_MII_TXD2 ---------------------> PC2
      ETH_MII_TXD3 ---------------------> PB8
      ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1
      ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7
      ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4
      ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5
      ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11
      ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13
      ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14
    */

   /* Configure PA1, PA2 and PA7 */
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_7;
   GPIO_Init(GPIOA, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_ETH);

   /* Configure PB5 and PB8 */
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_8;
   GPIO_Init(GPIOB, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_ETH);	
   GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_ETH);

   /* Configure PC1, PC2, PC3, PC4 and PC5 */
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5;
   GPIO_Init(GPIOC, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource1, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource2, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource3, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource4, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource5, GPIO_AF_ETH);
                                
   /* Configure PG11, PG14 and PG13 */
   GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_11 | GPIO_Pin_13 | GPIO_Pin_14;
   GPIO_Init(GPIOG, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOG, GPIO_PinSource11, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOG, GPIO_PinSource13, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOG, GPIO_PinSource14, GPIO_AF_ETH);

   /* Configure PH2, PH3, PH6, PH7 */
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_6 | GPIO_Pin_7;
   GPIO_Init(GPIOH, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOH, GPIO_PinSource2, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOH, GPIO_PinSource3, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOH, GPIO_PinSource6, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOH, GPIO_PinSource7, GPIO_AF_ETH);

   /* Configure PI10 */
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
   GPIO_Init(GPIOI, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOI, GPIO_PinSource10, GPIO_AF_ETH);
     
} /* GPIO_Config_MB786 */
#endif /* USE_BOARD_MB786 */

#if defined(USE_BOARD_E407)
/*************************************************************************/
/*  GPIO_Config_E407                                                     */
/*                                                                       */
/*  Configures the GPIO ports.                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void GPIO_Config_E407 (void)
{
   GPIO_InitTypeDef GPIO_InitStructure;

   /* Enable GPIOs clocks */
   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | 
                          RCC_AHB1Periph_GPIOC | 
                          RCC_AHB1Periph_GPIOG, ENABLE);

   /* Enable SYSCFG clock */
   RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
   
   /* RMII Media interface */
   RCC_AHB1PeriphResetCmd(RCC_AHB1Periph_ETH_MAC, ENABLE);
   SYSCFG_ETH_MediaInterfaceConfig(SYSCFG_ETH_MediaInterface_RMII);
  
   /* Ethernet pins configuration
   
      ETH_MDIO -------------------------> PA2
      ETH_MDC --------------------------> PC1
      ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1
      ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7
      ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4
      ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5
      ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11
      ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13
      ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14
    */

   /* Configure PA1, PA2 and PA7 */
   GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_7;
   GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
   GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_Init(GPIOA, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_ETH);

   /* Configure PC1, PC4 and PC5 */
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5;
   GPIO_Init(GPIOC, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource1, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource4, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource5, GPIO_AF_ETH);
                                
   /* Configure PG11, PG13 and PG14 */
   GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_11 | GPIO_Pin_13 | GPIO_Pin_14;
   GPIO_Init(GPIOG, &GPIO_InitStructure);
   GPIO_PinAFConfig(GPIOG, GPIO_PinSource11, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOG, GPIO_PinSource13, GPIO_AF_ETH);
   GPIO_PinAFConfig(GPIOG, GPIO_PinSource14, GPIO_AF_ETH);
     
} /* GPIO_Config_E407 */
#endif /* USE_BOARD_E407 */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  BoardETHConfig                                                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: NULL / PHYDriver                                             */
/*************************************************************************/
IP_PHY_DRIVER *BoardETHConfig (void)
{
   IP_PHY_DRIVER *pPHY;
   
   /* 
    * Configure the GPIO ports for ethernet pins.
    * This is a macro depending of the board.
    */
   GPIO_CONFIG();
   
   /* 
    * Configure the Ethernet MAC/DMA 
    */
   MACDMA_Config();
   
   /*
    * Find the PHY which is used on the board.
    */
   pPHY = FindPHY();
   if (pPHY != NULL)
   {
      /* 
       * Configure Ethernet interrupt
       */
      NVIC_SetPriority(ETH_IRQn, 3);
      NVIC_EnableIRQ(ETH_IRQn);
   }      

   return(pPHY);
} /* BoardETHConfig */

/*** EOF ***/
